@@ -14,9 +14,10 @@ from .._compat import (
     iterkeys,
     itervalues,
     iteritems,
+    to_bytes,
     long,
 )
-from .._compat import to_bytes
+
 from .._globals import THREAD_LOCAL
 from .serializers import serializers
 
@@ -414,6 +415,9 @@ class FakeCursor(object):
     def __setattr__(self, attr, value):
         self.warn_bad_usage(attr)
 
+    def close(self):
+        return
+
 
 class NullCursor(FakeCursor):
     lastrowid = 1
@@ -537,7 +541,7 @@ class DatabaseStoredFile:
         return self.read(bytes)
 
     def readline(self):
-        i = self.data.find("\n", self.p) + 1
+        i = self.data.find(b"\n", self.p) + 1
         if i > 0:
             data, self.p = self.data[self.p : i], i
         else:
@@ -552,7 +556,7 @@ class DatabaseStoredFile:
             self.db.executesql(
                 "DELETE FROM web2py_filesystem WHERE path='%s'" % self.filename
             )
-            query = "INSERT INTO web2py_filesystem(path,content) VALUES ('%s','%s')"
+            query = "INSERT INTO web2py_filesystem(path,content) VALUES (%s, %s)"
             args = (to_bytes(self.filename), self.data)
             self.db.executesql(query, args)
             self.db.commit()
